#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <GL/glut.h>

#ifndef M_PI
#define M_PI 3.1415926535897932384626433832795
#endif

#define RAD(x)          (M_PI*(x)/180)
#define GRAUS(x)        (180*(x)/M_PI)

#define DEBUG           0
#define LARGURA_CAMPO   400
#define ALTURA_CAMPO    200
#define LARGURA_RAQUETES 10
#define ALTURA_RAQUETES  50
#define LINHA_SERVICO   50
#define VELOCIDADE_BOLA 8
#define VELOCIDADE_RAQUETES 15
#define TAMANHO_BOLA    10
#define DELAY_MOVIMENTO 30
#define DELAY_TECLAS    20

/* VARIAVEIS GLOBAIS */

typedef struct {
  GLboolean   q,a,p,l;
}Teclas;

typedef struct {
  GLboolean   doubleBuffer;
  GLint       delayMovimento;
  GLint       delayTeclas;
  Teclas      teclas;
  GLuint      menu_vel_bola_id;
  GLuint      menu_tam_bola_id;
  GLuint      menu_vel_raq_id;
  GLuint      menu_tam_raq_id;
  GLuint      menu_id;
  GLboolean   menuActivo;
  GLboolean   debug;
}Estado;

typedef struct {
  GLfloat       x,y;
  GLint         pontuacao;
}Raquete;

typedef struct {
  GLfloat       x,y;
  GLint         velocidade;
  GLfloat       direccao;
  GLint       tamanho;
}Bola;

typedef struct {
  Raquete     jogador1,jogador2;
  Bola        bola;
  GLint       alturaRaquetes;
  GLint       velocidadeRaquetes;
  GLint       servico;
  GLboolean   parado;
}Modelo;

Estado estado;
Modelo modelo;

void inicia_jogo()
{

  modelo.servico=(modelo.servico==2) ? 1 : 2;
  
  modelo.jogador1.x=LINHA_SERVICO+LARGURA_RAQUETES*.5;
  modelo.jogador2.x=LARGURA_CAMPO-(LINHA_SERVICO+LARGURA_RAQUETES*.5);
  modelo.jogador1.y=modelo.jogador2.y=ALTURA_CAMPO*.5;
  modelo.bola.x=LARGURA_CAMPO*.5;
  modelo.bola.y=ALTURA_CAMPO*.5;

    modelo.bola.direccao=RAD(rand()%120-60);
  if(modelo.servico==2)
    modelo.bola.direccao=M_PI-modelo.bola.direccao;

  if(modelo.bola.direccao<0)
    modelo.bola.direccao+=2*M_PI;

}

/* Inicializao do ambiente OPENGL */
void Init(void)
{

  srand( (unsigned)time( NULL ) );

  modelo.bola.velocidade=VELOCIDADE_BOLA;
  modelo.bola.tamanho=TAMANHO_BOLA;
  modelo.alturaRaquetes=ALTURA_RAQUETES;
  modelo.velocidadeRaquetes=VELOCIDADE_RAQUETES;
  modelo.servico=2;
  modelo.jogador1.pontuacao=modelo.jogador2.pontuacao=0;
  modelo.parado=GL_FALSE;

  estado.menuActivo=GL_FALSE;
  estado.delayMovimento=DELAY_MOVIMENTO;
  estado.delayTeclas=DELAY_TECLAS;
  
  estado.teclas.a=estado.teclas.q=estado.teclas.l=estado.teclas.p=GL_FALSE;

  inicia_jogo();
    
  glClearColor(0.0, 0.0, 0.0, 0.0);

  glEnable(GL_POINT_SMOOTH);
  glEnable(GL_LINE_SMOOTH);
  glEnable(GL_POLYGON_SMOOTH);

  glutIgnoreKeyRepeat(GL_TRUE);

}

/**************************************
***  callbacks de janela/desenho    ***
**************************************/

// CALLBACK PARA REDIMENSIONAR JANELA

void Reshape(int width, int height)
{
  // glViewport(botom, left, width, height)
  // define parte da janela a ser utilizada pelo OpenGL

  glViewport(0, 0, (GLint) width, (GLint) height);
  

  // Matriz Projeccao
  // Matriz onde se define como o mundo e apresentado na janela
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  // gluOrtho2D(left,right,bottom,top); 
  // projeccao ortogonal 2D, com profundidade (Z) entre -1 e 1
  gluOrtho2D(0, LARGURA_CAMPO, 0, ALTURA_CAMPO);

  // Matriz Modelview
  // Matriz onde so realizadas as tranformacoes dos modelos desenhados
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}


void strokeString(char *str,double x, double y, double z, double s)
{
	int i,n;
	
	n = strlen(str);
	glPushMatrix();
	glColor3d(0.0, 0.0, 0.0);
	glTranslated(x,y,z);
	glScaled(s,s,s);
	for(i=0;i<n;i++)
		glutStrokeCharacter(GLUT_STROKE_ROMAN,(int)str[i]);

	glPopMatrix();

}

void bitmapString(char *str, double x, double y)
{
	int i,n;
  
  // fonte pode ser:
  // GLUT_BITMAP_8_BY_13
  // GLUT_BITMAP_9_BY_15
  // GLUT_BITMAP_TIMES_ROMAN_10
  // GLUT_BITMAP_TIMES_ROMAN_24
  // GLUT_BITMAP_HELVETICA_10
  // GLUT_BITMAP_HELVETICA_12
  // GLUT_BITMAP_HELVETICA_18
  //
  // int glutBitmapWidth  	(	void *font , int character);
  // devolve a largura de um carcter
  //
  // int glutBitmapLength 	(	void *font , const unsigned char *string );
  // devolve a largura de uma string (soma da largura de todos os caracteres)

	n = strlen(str);
	glRasterPos2d(x,y);
	for (i=0;i<n;i++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,(int)str[i]);
}

void bitmapCenterString(char *str, double x, double y)
{
	int i,n;
  
	n = strlen(str);
	glRasterPos2d(x-glutBitmapLength(GLUT_BITMAP_HELVETICA_18,(const unsigned char *)str)*0.5,y);
	for (i=0;i<n;i++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,(int)str[i]);
}


void desenha_campo(int p1, int p2)
{
  char str[255];

  glLineWidth(3);
  glLineStipple(2,0xFF00);
  
  glEnable(GL_LINE_STIPPLE);
  glBegin(GL_LINES);
    glColor3f(1.0f ,1.0f ,1.0f);
    glVertex2f(LARGURA_CAMPO*.5,0);
    glVertex2f(LARGURA_CAMPO*.5,ALTURA_CAMPO);
  glEnd();
  glDisable(GL_LINE_STIPPLE);

  sprintf(str,"%d",p1);
  bitmapString(str, LARGURA_CAMPO*.4,ALTURA_CAMPO*.9);
  sprintf(str,"%d",p2);
  bitmapString(str, LARGURA_CAMPO*.6,ALTURA_CAMPO*.9);

  if(estado.debug)
  {
    sprintf(str,"ang:%.1f*PI\n x:%.1f\n y:%.1f ",modelo.bola.direccao/M_PI,modelo.bola.x,modelo.bola.y);
    bitmapCenterString(str, LARGURA_CAMPO*.1,ALTURA_CAMPO*.1);
  }

}


void desenha_raquete(Raquete raq)
{
  glBegin(GL_POLYGON);
    glVertex2f(raq.x-LARGURA_RAQUETES*.5,raq.y-modelo.alturaRaquetes*.5);
    glVertex2f(raq.x+LARGURA_RAQUETES*.5,raq.y-modelo.alturaRaquetes*.5);
    glVertex2f(raq.x+LARGURA_RAQUETES*.5,raq.y+modelo.alturaRaquetes*.5);
    glVertex2f(raq.x-LARGURA_RAQUETES*.5,raq.y+modelo.alturaRaquetes*.5);
  glEnd();
}


void desenha_bola(Bola bola)
{

  glPointSize(modelo.bola.tamanho);

  glBegin(GL_POINTS);
    glVertex2f(bola.x,bola.y);
  glEnd();
}


// ... definicao das rotinas auxiliares de desenho ...

// Callback de desenho

void Draw(void)
{
  glClear(GL_COLOR_BUFFER_BIT);
// ... chamada das rotinas auxiliares de desenho ...

  desenha_campo(modelo.jogador1.pontuacao, modelo.jogador2.pontuacao);

  glColor3f(1.0f, 0.0f, 0.0f);
  desenha_raquete(modelo.jogador1);

  glColor3f(0.0f, 0.0f, 1.0f);
  desenha_raquete(modelo.jogador2);

  glColor3f(1.0f, 1.0f, 0.0f);
  desenha_bola(modelo.bola);

  if(modelo.parado)
  {
      glColor3f(0.7,0.7,0.7);
      bitmapCenterString("Jogo Parado, use o menu para continuar", LARGURA_CAMPO*.5,ALTURA_CAMPO*.5);
  }

  
  glFlush();
  if (estado.doubleBuffer)
    glutSwapBuffers();
}

/*******************************
***   callbacks timer/idle   ***
*******************************/

void Timer(int value)
{
  glutTimerFunc(estado.delayMovimento, Timer, 0);
  // ... accoes do temporizador ... 
  
  if(estado.menuActivo || modelo.parado)
    return;

  // movimenta bola
  modelo.bola.x+=modelo.bola.velocidade*cos(modelo.bola.direccao);
  modelo.bola.y+=modelo.bola.velocidade*sin(modelo.bola.direccao);

  // testa fim de ponto

  if(modelo.bola.x-modelo.bola.tamanho<0)
  {
    modelo.jogador2.pontuacao++;
    inicia_jogo();
  }

  if(modelo.bola.x+modelo.bola.tamanho>LARGURA_CAMPO)
  {
    modelo.jogador1.pontuacao++;
    inicia_jogo();
  }

  // testa coliso paredes

  if(modelo.bola.y-modelo.bola.tamanho<0 || modelo.bola.y+modelo.bola.tamanho>ALTURA_CAMPO)
  {
    modelo.bola.direccao=2*M_PI-modelo.bola.direccao;
  }

  // testa coliso raquete 1

  if(modelo.bola.direccao<1.5*M_PI && modelo.bola.direccao>0.5*M_PI) // bola vai para a esquerda
  {
    if(modelo.bola.x-modelo.bola.tamanho<modelo.jogador1.x+LARGURA_RAQUETES*0.5 && 
       modelo.bola.x>modelo.jogador1.x-LARGURA_RAQUETES*0.5 &&
       modelo.bola.y+modelo.bola.tamanho>modelo.jogador1.y-modelo.alturaRaquetes*0.5 &&
       modelo.bola.y-modelo.bola.tamanho<modelo.jogador1.y+modelo.alturaRaquetes*0.5) 
          modelo.bola.direccao=M_PI-modelo.bola.direccao;
  }
  else // bola vai para a direita
  {
   if(modelo.bola.x<modelo.jogador2.x+LARGURA_RAQUETES*0.5 && 
      modelo.bola.x+modelo.bola.tamanho>modelo.jogador2.x-LARGURA_RAQUETES*0.5 &&
      modelo.bola.y+modelo.bola.tamanho>modelo.jogador2.y-modelo.alturaRaquetes*0.5 &&
      modelo.bola.y-modelo.bola.tamanho<modelo.jogador2.y+modelo.alturaRaquetes*0.5 
      )
          modelo.bola.direccao=M_PI-modelo.bola.direccao;
  }

  if(modelo.bola.direccao<0)
    modelo.bola.direccao+=2*M_PI;
  if(modelo.bola.direccao>2*M_PI)
    modelo.bola.direccao-=2*M_PI;


  // testa coliso raquete 2

  // redesenhar o ecra 
  glutPostRedisplay();
}


void TimerTeclas(int value)
{
  glutTimerFunc(estado.delayTeclas, TimerTeclas, 0);
  // ... accoes do temporizador ... 

  if(estado.menuActivo || modelo.parado)
    return;

  if(estado.teclas.a && modelo.jogador1.y>modelo.alturaRaquetes*.5)
  {
     modelo.jogador1.y-=modelo.velocidadeRaquetes;
     if(modelo.jogador1.y<modelo.alturaRaquetes*.5)
       modelo.jogador1.y=modelo.alturaRaquetes*.5;
  }
  if(estado.teclas.q && modelo.jogador1.y<ALTURA_CAMPO-modelo.alturaRaquetes*.5)
  {
     modelo.jogador1.y+=modelo.velocidadeRaquetes;
     if(modelo.jogador1.y>ALTURA_CAMPO-modelo.alturaRaquetes*.5)
       modelo.jogador1.y=ALTURA_CAMPO-modelo.alturaRaquetes*.5;
  }
  if(estado.teclas.l && modelo.jogador2.y>modelo.alturaRaquetes*.5)
  {
     modelo.jogador2.y-=modelo.velocidadeRaquetes;
     if(modelo.jogador2.y<modelo.alturaRaquetes*.5)
       modelo.jogador2.y=modelo.alturaRaquetes*.5;
  }
  if(estado.teclas.p && modelo.jogador2.y<ALTURA_CAMPO-modelo.alturaRaquetes*.5)
  {
     modelo.jogador2.y+=modelo.velocidadeRaquetes;
     if(modelo.jogador2.y>ALTURA_CAMPO-modelo.alturaRaquetes*.5)
       modelo.jogador2.y=ALTURA_CAMPO-modelo.alturaRaquetes*.5;
  }

  // redesenhar o ecra 
  glutPostRedisplay();
}

void imprime_ajuda(void)
{
  printf("\n\nDesenho de um quadrado\n");
  printf("h,H - Ajuda \n");
  printf("+   - Aumentar tamanho do Lado\n");
  printf("-   - Diminuir tamanho do Lado\n");
  printf("ESC - Sair\n");
}

/*******************************
***  callbacks de teclado    ***
*******************************/

// Callback para interaccao via teclado (carregar na tecla)

void Key(unsigned char key, int x, int y)
{
  switch (key) {
    case 27:
      exit(1);
    // ... accoes sobre outras teclas ... 

    case 'h' :
    case 'H' :
                imprime_ajuda();
            break;
    case 'i' :
    case 'I' :
                inicia_jogo();
            break;
    case 'Q' : 
    case 'q' : estado.teclas.q=GL_TRUE;
           break; 
    case 'A' : 
    case 'a' : estado.teclas.a=GL_TRUE;
           break;
    case 'P' : 
    case 'p' : estado.teclas.p=GL_TRUE;
           break;
    case 'L' : 
    case 'l' : estado.teclas.l=GL_TRUE;
           break;
    case 'D' : 
    case 'd' : estado.debug=!estado.debug;
               if(estado.menuActivo || modelo.parado)
                 glutPostRedisplay();
               printf("DEBUG is %s\n",(estado.debug)?"ON":"OFF");
           break;

  }

  if(estado.debug)
    printf("Carregou na tecla %c\n",key);

}

// Callback para interaccao via teclado (largar a tecla)

void KeyUp(unsigned char key, int x, int y)
{
  switch (key) {
    // ... accoes sobre outras teclas ... 

    case 'Q' : 
    case 'q' : estado.teclas.q=GL_FALSE;
           break; 
    case 'A' : 
    case 'a' : estado.teclas.a=GL_FALSE;
           break;
    case 'P' : 
    case 'p' : estado.teclas.p=GL_FALSE;
           break;
    case 'L' : 
    case 'l' : estado.teclas.l=GL_FALSE;
           break;

  }

  if(estado.debug)
    printf("Largou a tecla %c\n",key);
}

// Callback para interaccao via teclas especiais  (carregar na tecla)

/*void SpecialKey(int key, int x, int y)
{
  // ... accoes sobre outras teclas especiais ... 
  //    GLUT_KEY_F1 ... GLUT_KEY_F12
  //    GLUT_KEY_UP
  //    GLUT_KEY_DOWN
  //    GLUT_KEY_LEFT
  //    GLUT_KEY_RIGHT
  //    GLUT_KEY_PAGE_UP
  //    GLUT_KEY_PAGE_DOWN
  //    GLUT_KEY_HOME
  //    GLUT_KEY_END
  //    GLUT_KEY_INSERT 

  switch (key) {

  // redesenhar o ecra 
  //glutPostRedisplay();
  }


  if(estado.debug)
    printf("Carregou na tecla especial %d\n",key);
}/*/

// Callback para interaccao via teclas especiais (largar na tecla)

/*void SpecialKeyUp(int key, int x, int y)
{

  if(estado.debug)
    printf("Largou a tecla especial %d\n",key);

}/*/

void MenuStatus(int status, int x, int y)
{
  /* status => GLUT_MENU_IN_USE, GLUT_MENU_NOT_IN_USE 
     x,y    => coordenadas do ponteiro quando se entra no menu
  */

  if(status==GLUT_MENU_IN_USE) 
  {
    estado.menuActivo=GL_TRUE;
  } 
  else
  {
    estado.menuActivo=GL_FALSE;

  } 

  if(estado.debug)
    printf("MenuStatus status:%d coord:%d %d\n",status,x,y);

}

void menu(int opcao)
{
  char str[255];

  switch (opcao) {
      case 0 :
          exit(1);
        break;
      // velocidade bola
      case 1 :
          if(modelo.bola.velocidade>2)
          {
            modelo.bola.velocidade-=2;
            glutSetMenu(estado.menu_id); // altera entrada no menu inicial
            sprintf(str,"Velocidade da bola (%d)",modelo.bola.velocidade);
            glutChangeToSubMenu(2,str, estado.menu_vel_bola_id);
          }
        break;
      case 2 :
          if(modelo.bola.velocidade<16)
          {
            modelo.bola.velocidade+=2;
            glutSetMenu(estado.menu_id); // altera entrada no menu inicial
            sprintf(str,"Velocidade da bola (%d)",modelo.bola.velocidade);
            glutChangeToSubMenu(2,str, estado.menu_vel_bola_id);
          }
        break;
      // tamanho bola
      case 3 :
          if(modelo.bola.tamanho>5)
          {
            modelo.bola.tamanho--;
            glutSetMenu(estado.menu_id); // altera entrada no menu inicial
            sprintf(str,"Tamanho da bola (%d)",modelo.bola.tamanho);
            glutChangeToSubMenu(3, str, estado.menu_tam_bola_id);
          }
        break;
      case 4 :
          if(modelo.bola.tamanho<15)
          {
            modelo.bola.tamanho++;
            glutSetMenu(estado.menu_id); // altera entrada no menu inicial
            sprintf(str,"Tamanho da bola (%d)",modelo.bola.tamanho);
            glutChangeToSubMenu(3, str, estado.menu_tam_bola_id);
          }
       break;
      // velocidade raquete
      case 5 :
          if(modelo.velocidadeRaquetes>10)
          {
            modelo.velocidadeRaquetes-=5;
            glutSetMenu(estado.menu_id); // altera entrada no menu inicial
            sprintf(str,"Velocidade da raquete (%d)",modelo.velocidadeRaquetes);
            glutChangeToSubMenu(4, str, estado.menu_vel_raq_id);
          }
        break;
      case 6 :
          if(modelo.velocidadeRaquetes<30)
          {
            modelo.velocidadeRaquetes+=5;
            glutSetMenu(estado.menu_id); // altera entrada no menu inicial
            sprintf(str,"Velocidade da raquete (%d)",modelo.velocidadeRaquetes);
            glutChangeToSubMenu(4, str, estado.menu_vel_raq_id);
          }
        break;
      // tamanho raquete
      case 7 :
          if(modelo.alturaRaquetes>30)
          {
            modelo.alturaRaquetes-=10;
            glutSetMenu(estado.menu_id); // altera entrada no menu inicial
            sprintf(str,"Altura da raquete (%d)",modelo.alturaRaquetes);
            glutChangeToSubMenu(5, str,estado.menu_tam_raq_id);
          }
        break;
      case 8 :
          if(modelo.velocidadeRaquetes<100)
          {
            modelo.alturaRaquetes+=10;
            glutSetMenu(estado.menu_id); // altera entrada no menu inicial
            sprintf(str,"Altura da raquete (%d)",modelo.alturaRaquetes);
            glutChangeToSubMenu(5, str,estado.menu_tam_raq_id);
          }
        break;
      case 9 :
            modelo.parado=!modelo.parado;
            if(modelo.parado)
            {
              glutChangeToMenuEntry(1,"Comear",9);
              glutPostRedisplay();
            }
            else
            {
              glutChangeToMenuEntry(1,"Parar",9);
            }
        break;
  }

}

void cria_menu()
{
  char str[255];

  // funoes de manipulao de menus
  // id=glutCreateMenu(menu)  - cria um menu tratado pela funcao menu e devolve o id
  // glutDestroyMenu(id)      - destroi o menu identificado por id
  // id=glutGetMenu()         - devolve o id do menu actual
  // glutSetMenu(id)          - torna actual o menu com o id
  //
  // glutAddMenuEntry("texto", valor) -  adiciona uma entrada ao menu actual
  // glutChangeToMenuEntry(item, "novo texto", novo_valor) 
  //                                  -  altera a entrada item(1,2,3..) do menu actual
  // glutAddSubMenu("texto", id)      -  adiciona o submenu id ao menu actual
  // glutChangeToSubMenu(item, "novo texto", novo_id) 
  //                                  -  altera a entrada submenu item(1,2,3..) do menu actual
  // glutRemoveMenuItem(item)         - apaga a entrada item(1,2,3...)
  //
  // glutAttachMenu(botao)            - associa o menu actual ao botao 
  // glutDetachMenu(botao)            - desassocia o menu associado ao botao GLUT_RIGHT_BUTTON
  // botao = GLUT_LEFT_BUTTON, GLUT_MIDDLE_BUTTON ou GLUT_RIGHT_BUTTON

/* DEFINIR MENUS */
  estado.menu_vel_bola_id = glutCreateMenu(menu); // regista funo que trata do menu
  glutAddMenuEntry("Diminuir", 1);
  glutAddMenuEntry("Aumentar", 2);

  estado.menu_tam_bola_id = glutCreateMenu(menu); // regista funo que trata do menu
  glutAddMenuEntry("Diminuir", 3);
  glutAddMenuEntry("Aumentar", 4);

  estado.menu_vel_raq_id = glutCreateMenu(menu); // regista funo que trata do menu
  glutAddMenuEntry("Diminuir", 5);
  glutAddMenuEntry("Aumentar", 6);

  estado.menu_tam_raq_id = glutCreateMenu(menu); // regista funo que trata do menu
  glutAddMenuEntry("Diminuir", 7);
  glutAddMenuEntry("Aumentar", 8);

  estado.menu_id=glutCreateMenu(menu); // regista funo que trata do menu
  glutAddMenuEntry("Parar", 9);

  // adicionar submenus
  sprintf(str,"Velocidade da bola (%d)",modelo.bola.velocidade);
  glutAddSubMenu(str, estado.menu_vel_bola_id);
  sprintf(str,"Tamanho da bola (%d)",modelo.bola.tamanho);
  glutAddSubMenu(str, estado.menu_tam_bola_id);
  sprintf(str,"Velocidade da raquete (%d)",modelo.velocidadeRaquetes);
  glutAddSubMenu(str, estado.menu_vel_raq_id);
  sprintf(str,"Altura da raquete (%d)",modelo.alturaRaquetes);
  glutAddSubMenu(str,estado.menu_tam_raq_id);

  glutAddMenuEntry("Sair", 0);

  glutAttachMenu(GLUT_RIGHT_BUTTON); // Agarrar menu actual ao boto

}


void main(int argc, char **argv)
{
  estado.doubleBuffer=1;

  glutInit(&argc, argv);
  glutInitWindowPosition(0, 0);
  glutInitWindowSize(LARGURA_CAMPO, ALTURA_CAMPO);
  glutInitDisplayMode((estado.doubleBuffer) ? GLUT_DOUBLE : GLUT_SINGLE | GLUT_RGB);
  if (glutCreateWindow("Exemplo") == GL_FALSE)
    exit(1);

  Init();

  imprime_ajuda();

// Registar callbacks do GLUT
  
  // callbacks de janelas/desenho
  glutReshapeFunc(Reshape);
  glutDisplayFunc(Draw);

  // Callbacks de teclado
  glutKeyboardFunc(Key);
  glutKeyboardUpFunc(KeyUp);
  //glutSpecialFunc(SpecialKey);
  //glutSpecialUpFunc(SpecialKeyUp);

  // callbacks timer/idle
  glutTimerFunc(estado.delayMovimento, Timer, 0);
  glutTimerFunc(estado.delayTeclas, TimerTeclas, 0);

  cria_menu();
  glutMenuStatusFunc(MenuStatus);
  
  // COMECAR...
  glutMainLoop();
}
